__name__ = "lps_maestro"

__all__ = []

from . import authenticate
__all__.extend(authenticate.__all__)
from .authenticate import *

from . import castor
__all__.extend(castor.__all__)
from .castor import *

from . import constants
__all__.extend(constants.__all__)
from .constants import *

from . import task
__all__.extend(task.__all__)
from .task import *

from . import utils
__all__.extend(utils.__all__)
from .utils import *

# Checking for new versions
import pkg_resources
import requests
def check(name):
    # Getting current version
    current_version = pkg_resources.get_distribution('lps_maestro').version
    # Getting latest version
    r = requests.get('https://pypi.org/pypi/lps-maestro/json')
    d = r.json()
    latest_version = list(d['releases'].keys())[-1]
    if latest_version == current_version:
        return True, latest_version, current_version
    else:
        return False, latest_version, current_version

print (" Welcome to LPS Maestro!")
print (" - Checking for updates...")
try:
    v = check('lps-maestro')
    if (v[0] == True):
        print (" - This is the latest version available! (current={}, latest={})".format(v[1], v[2]))
    else:
        print (" => Version {} is available, please update over your {}!".format(v[1], v[2]))
except:
    print (" - Failed to check for updates")


__all__ = [
  'castor'
]

import os
import sys
import argparse
import requests
import pickle
import base64
from pathlib import Path
from lps_maestro.utils import getCredentialsData, decode_base64
from lps_maestro.constants import *

class Castor ():

  def list( self, username, cli=False ):

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'credentials':credentials
    }
    try:
      if cli:
        r = requests.post(url='http://146.164.147.170:5020/list-datasets', data=data)
        print (r.json()['message'])
      else:
        r = requests.post(url='http://146.164.147.170:5020/list-datasets-py', data=data)
        try:
          return r.json()
        except:
          pickled_response = r.content.decode('utf-8')
          df = pickle.loads(decode_base64(pickled_response.encode()))
          return df
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def delete( self, datasetname ):

    if datasetname.split('.')[0] != 'user':
      print( 'The dataset name must start with "user.<username>.taskname."')
      raise BaseException
    username = datasetname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'datasetname':datasetname,
      'credentials':credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/delete-dataset', data=data)
      print (r.text)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def download( self, datasetname ):

    if datasetname.split('.')[0] != 'user':
      print ( 'The dataset name must start with "user.<username>.taskname."')
    username = datasetname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'datasetname':datasetname,
      'credentials':credentials
    }

    try:
      r = requests.post(url='http://146.164.147.170:5020/download-dataset', data=data)
      try:
        if r.json()['message'] == "Internal Server Error":
          print (r.text)
          return
        error_code = r.json()['error_code']
        print (r.text)
      except:
        with open('./{}'.format('{}.zip'.format(datasetname)), 'wb') as f:
          f.write(r.content)
    except requests.exceptions.ConnectionError:
      print ("Failed to connect to LPS Cluster.")

  def upload( self , datasetname, path ):

    if datasetname.split('.')[0] != 'user':
      print( 'The dataset name must start with "user.<username>.taskname."')
      raise BaseException
    username = datasetname.split('.')[1]

    credentials = getCredentialsData()
    if credentials == False:
      return

    data = {
      'username':username,
      'datasetname':datasetname,
      'credentials':credentials
    }

    # Parsing input
    file_list = []
    if (os.path.isdir(path)):
      for i in os.listdir(path):
        if os.path.isfile(os.path.join(path, i)):
          file_list.append (os.path.join(path, i))
      if (not file_list):
        print ("File does not exist.")
    else:
      if (not os.path.exists(path)):
        print ("File does not exist.")
      else:
        file_list.append(path)
    
    for filename in file_list:
      fin = open(filename, 'rb')
      files = {'file':fin}
      try:
        r = requests.post(url='http://146.164.147.170:5020/upload-dataset', data=data, files=files)
        print (r.text)
      except requests.exceptions.ConnectionError:
        print ("Failed to connect to LPS Cluster.")
      finally:
        fin.close()

castor = Castor()